@@ -122,7 +122,7 @@ polkit_permission_constructed (GObject *object)
   PolkitPermission *permission = POLKIT_PERMISSION (object);
 
   if (permission->subject == NULL)
-    permission->subject = polkit_unix_process_new (getpid ());
+    permission->subject = polkit_unix_process_new_for_owner (getpid (), 0, getuid ());
 
   if (G_OBJECT_CLASS (polkit_permission_parent_class)->constructed != NULL)
     G_OBJECT_CLASS (polkit_permission_parent_class)->constructed (object);
@@ -535,9 +535,6 @@ acquire_cb (GObject      *source_object,
   AcquireData *data = user_data;
   PolkitAuthorizationResult *result;
   GError *error;
-  gboolean ret;
-
-  ret = FALSE;
 
   error = NULL;
   result = polkit_authority_check_authorization_finish (data->permission->authority,
@@ -552,11 +549,22 @@ acquire_cb (GObject      *source_object,
       process_result (data->permission, result);
       if (!polkit_authorization_result_get_is_authorized (result))
         {
-          g_simple_async_result_set_error (data->simple,
-                                           POLKIT_ERROR,
-                                           POLKIT_ERROR_FAILED,
-                                           "Failed to acquire permission for action-id %s",
-                                           data->permission->action_id);
+          if (polkit_authorization_result_get_dismissed (result))
+            {
+              g_simple_async_result_set_error (data->simple,
+                                               G_IO_ERROR,
+                                               G_IO_ERROR_CANCELLED,
+                                               "User dismissed authentication dialog while trying to acquire permission for action-id %s",
+                                               data->permission->action_id);
+            }
+          else
+            {
+              g_simple_async_result_set_error (data->simple,
+                                               POLKIT_ERROR,
+                                               POLKIT_ERROR_FAILED,
+                                               "Failed to acquire permission for action-id %s",
+                                               data->permission->action_id);
+            }
         }
       g_object_unref (result);
     }
@@ -640,6 +648,14 @@ acquire (GPermission   *gpermission,
         {
           ret = TRUE;
         }
+      else if (polkit_authorization_result_get_dismissed (result))
+        {
+          g_set_error (error,
+                       G_IO_ERROR,
+                       G_IO_ERROR_CANCELLED,
+                       "User dismissed authentication dialog while trying to acquire permission for action-id %s",
+                       permission->action_id);
+        }
       else
         {
           g_set_error (error,
