@@ -50,9 +50,9 @@ base_init (gpointer g_iface)
 GType
 polkit_subject_get_type (void)
 {
-  static GType iface_type = 0;
+  static volatile gsize g_define_type_id__volatile = 0;
 
-  if (iface_type == 0)
+  if (g_once_init_enter (&g_define_type_id__volatile))
     {
       static const GTypeInfo info =
       {
@@ -68,12 +68,14 @@ polkit_subject_get_type (void)
         NULL                    /* value_table    */
       };
 
-      iface_type = g_type_register_static (G_TYPE_INTERFACE, "PolkitSubject", &info, 0);
+      GType iface_type =
+        g_type_register_static (G_TYPE_INTERFACE, "PolkitSubject", &info, 0);
 
       g_type_interface_add_prerequisite (iface_type, G_TYPE_OBJECT);
+      g_once_init_leave (&g_define_type_id__volatile, iface_type);
     }
 
-  return iface_type;
+  return g_define_type_id__volatile;
 }
 
 /**
@@ -218,8 +220,8 @@ polkit_subject_exists_sync   (PolkitSubject  *subject,
  * Creates an object from @str that implements the #PolkitSubject
  * interface.
  *
- * Returns: A #PolkitSubject or %NULL if @error is set. Free with
- * g_object_unref().
+ * Returns: (transfer full): A #PolkitSubject or %NULL if @error is
+ * set. Free with g_object_unref().
  */
 PolkitSubject *
 polkit_subject_from_string  (const gchar   *str,
@@ -238,13 +240,22 @@ polkit_subject_from_string  (const gchar   *str,
     {
       gint scanned_pid;
       guint64 scanned_starttime;
-      if (sscanf (str, "unix-process:%d:%" G_GUINT64_FORMAT, &scanned_pid, &scanned_starttime) == 2)
+      gint scanned_uid;
+      if (sscanf (str, "unix-process:%d:%" G_GUINT64_FORMAT ":%d", &scanned_pid, &scanned_starttime, &scanned_uid) == 3)
         {
+          subject = polkit_unix_process_new_for_owner (scanned_pid, scanned_starttime, scanned_uid);
+        }
+      else if (sscanf (str, "unix-process:%d:%" G_GUINT64_FORMAT, &scanned_pid, &scanned_starttime) == 2)
+        {
+	  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
           subject = polkit_unix_process_new_full (scanned_pid, scanned_starttime);
+	  G_GNUC_END_IGNORE_DEPRECATIONS
         }
       else if (sscanf (str, "unix-process:%d", &scanned_pid) == 1)
         {
-          subject = polkit_unix_process_new_full (scanned_pid, 0);
+	  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+          subject = polkit_unix_process_new (scanned_pid);
+	  G_GNUC_END_IGNORE_DEPRECATIONS
           if (polkit_unix_process_get_start_time (POLKIT_UNIX_PROCESS (subject)) == 0)
             {
               g_object_unref (subject);
@@ -297,6 +308,8 @@ polkit_subject_to_gvariant (PolkitSubject *subject)
                              g_variant_new_uint32 (polkit_unix_process_get_pid (POLKIT_UNIX_PROCESS (subject))));
       g_variant_builder_add (&builder, "{sv}", "start-time",
                              g_variant_new_uint64 (polkit_unix_process_get_start_time (POLKIT_UNIX_PROCESS (subject))));
+      g_variant_builder_add (&builder, "{sv}", "uid",
+                             g_variant_new_int32 (polkit_unix_process_get_uid (POLKIT_UNIX_PROCESS (subject))));
     }
   else if (POLKIT_IS_UNIX_SESSION (subject))
     {
@@ -395,6 +408,7 @@ polkit_subject_new_for_gvariant (GVariant  *variant,
       GVariant *v;
       guint32 pid;
       guint64 start_time;
+      gint32 uid;
 
       v = lookup_asv (details_gvariant, "pid", G_VARIANT_TYPE_UINT32, error);
       if (v == NULL)
@@ -414,7 +428,18 @@ polkit_subject_new_for_gvariant (GVariant  *variant,
       start_time = g_variant_get_uint64 (v);
       g_variant_unref (v);
 
-      ret = polkit_unix_process_new_full (pid, start_time);
+      v = lookup_asv (details_gvariant, "uid", G_VARIANT_TYPE_INT32, NULL);
+      if (v != NULL)
+        {
+          uid = g_variant_get_int32 (v);
+          g_variant_unref (v);
+        }
+      else
+        {
+          uid = -1;
+        }
+
+      ret = polkit_unix_process_new_for_owner (pid, start_time, uid);
     }
   else if (g_strcmp0 (kind, "unix-session") == 0)
     {
