@@ -44,7 +44,7 @@ help (void)
 "  -d, --details=KEY VALUE            Add (KEY, VALUE) to information about the action\n"
 "  --enable-internal-agent            Use an internal authentication agent if necessary\n"
 "  --list-temp                        List temporary authorizations for current session\n"
-"  -p, --process=PID[,START_TIME]     Check authorization of specified process\n"
+"  -p, --process=PID[,START_TIME,UID] Check authorization of specified process\n"
 "  --revoke-temp                      Revoke all temporary authorizations for current session\n"
 "  -s, --system-bus-name=BUS_NAME     Check authorization of owner of BUS_NAME\n"
 "  --version                          Show version\n"
@@ -382,6 +382,7 @@ main (int argc, char *argv[])
       else if (g_strcmp0 (argv[n], "--process") == 0 || g_strcmp0 (argv[n], "-p") == 0)
         {
           gint pid;
+	  guint uid;
           guint64 pid_start_time;
 
           n++;
@@ -392,13 +393,21 @@ main (int argc, char *argv[])
               goto out;
             }
 
-          if (sscanf (argv[n], "%i,%" G_GUINT64_FORMAT, &pid, &pid_start_time) == 2)
+          if (sscanf (argv[n], "%i,%" G_GUINT64_FORMAT ",%u", &pid, &pid_start_time, &uid) == 3)
             {
+              subject = polkit_unix_process_new_for_owner (pid, pid_start_time, uid);
+            }
+          else if (sscanf (argv[n], "%i,%" G_GUINT64_FORMAT, &pid, &pid_start_time) == 2)
+            {
+	      G_GNUC_BEGIN_IGNORE_DEPRECATIONS
               subject = polkit_unix_process_new_full (pid, pid_start_time);
+	      G_GNUC_END_IGNORE_DEPRECATIONS
             }
           else if (sscanf (argv[n], "%i", &pid) == 1)
             {
+	      G_GNUC_BEGIN_IGNORE_DEPRECATIONS
               subject = polkit_unix_process_new (pid);
+	      G_GNUC_END_IGNORE_DEPRECATIONS
             }
           else
             {
